use std::path::PathBuf;

use ::compiler::read_file;
use ::lexer::Token;
use ::lexer::TokenTypes;
use ::reader::Reader;
use ::util::Util;



///
pub struct Parser<'a>
{
   ///
   util: &'a Util
}



///
fn flush_buffer(output: &mut Vec<String>, buffer: &mut String)
{
   // Put the buffer into the output list.
   output.push(buffer.clone());

   // Clear the buffer out.
   buffer.clear();
}

///
fn buffer_token(buffer: &mut String, token: &Token)
{
   buffer.push_str(&token.to_string());
}

///
fn is_start_of_code_section(tokens: &Vec<Token>, pos: &usize) -> bool
{
   // Check if this is the start of a code section.
   if *pos + 1usize < tokens.len() &&
      tokens[*pos].get_type() == TokenTypes::Symbol &&
      tokens[*pos + 1usize].get_type() == TokenTypes::Symbol &&
      tokens[*pos].to_string() == "{" &&
      tokens[*pos + 1usize].to_string() == "@"
   {
      return true;
   }

   false
}

///
fn is_end_of_code_section(tokens: &Vec<Token>, pos: &usize) -> bool
{
   // Check if this is the start of a code section.
   if *pos + 1usize < tokens.len() &&
      tokens[*pos].get_type() == TokenTypes::Symbol &&
      tokens[*pos + 1usize].get_type() == TokenTypes::Symbol &&
      tokens[*pos].to_string() == "@" &&
      tokens[*pos + 1usize].to_string() == "}"
   {
      return true;
   }

   false
}

///
fn is_include_statement(tokens: &Vec<Token>, pos: &usize) -> bool
{
   // Check if this is an include statement.
   if *pos + 4usize < tokens.len() &&
      tokens[*pos + 0usize].get_type() == TokenTypes::Keyword &&
      tokens[*pos + 1usize].get_type() == TokenTypes::Symbol &&
      tokens[*pos + 2usize].get_type() == TokenTypes::StaticString &&
      tokens[*pos + 3usize].get_type() == TokenTypes::Symbol &&
      tokens[*pos + 4usize].get_type() == TokenTypes::Symbol &&
      tokens[*pos + 0usize].to_string() == "include" &&
      tokens[*pos + 1usize].to_string() == "(" &&
      tokens[*pos + 3usize].to_string() == ")" &&
      tokens[*pos + 4usize].to_string() == ";"
   {
      return true;
   }

   false
}



impl<'a> Parser<'a>
{
   ///
   pub fn new(util_ptr: &'a Util) -> Parser
   {
      Parser
      {
         util: util_ptr
      }
   }

   ///
   pub fn parse(&mut self, tokens: Vec<Token>) -> Vec<String>
   {
      let mut is_in_code_block: bool;
      let mut position: usize;
      let mut include_file: PathBuf;
      let mut buffer: String;
      let mut output: Vec<String>;

      // Create a new set of lines to hold the parsed data.
      output = Vec::new();

      // Setup the String buffer.
      buffer = String::new();

      // Parse the tokens passed in.
      is_in_code_block = false;
      position = 0usize;
      while position < tokens.len()
      {
         if is_in_code_block == true
         {
            if is_end_of_code_section(&tokens, &position) == true
            {
               // Flush any previous buffer.
               flush_buffer(&mut output, &mut buffer);

               // Mark that we are no longer in a code block.
               is_in_code_block = false;

               // Two tokens have been read.
               position += 2;

               println!("{}: Is end of code block.", position);
            }
            else if is_include_statement(&tokens, &position) == true
            {
               // Flush any previous buffer.
               flush_buffer(&mut output, &mut buffer);

               // Convert the String portion to a PathBuf.
               include_file =
                  PathBuf::from(
                     tokens[position + 2usize].to_string().replace("\"", ""));

               // Try to find the include file in one of the
               // include directories.
               match self.util.get_include_file_path(&include_file)
               {
                  Some(file) =>
                  {
                     // Process the include file found and add it to
                     // the output list.
                     output.push(read_file(self.util, &file));
                  }

                  None =>
                  {
                     error!("Unable to find desired include file:  {:?}",
                            include_file);
                  }
               }

               // Five tokens have been read.
               position += 5;

               println!("{}: Is include statement.", position);
            }
            else
            {
               // This token is nothing interesting, so
               // process it generally and move the position forward.
               buffer_token(&mut buffer, &tokens[position]);

               position += 1;
            }
         }
         else
         {
            if is_start_of_code_section(&tokens, &position) == true
            {
               // Flush any previous buffer.
               flush_buffer(&mut output, &mut buffer);

               // Mark that we are in a code block.
               is_in_code_block = true;

               // Two tokens have been read.
               position += 2;

               println!("{}: Is start of code block.", position);
            }
            else
            {
               // This token is nothing interesting, so
               // process it generally and move the position forward.
               buffer_token(&mut buffer, &tokens[position]);

               position += 1;
            }
         }
      }

      // Flush any left over buffer.
      flush_buffer(&mut output, &mut buffer);

      // Return the parsed input.
      output
   }
}
