#[macro_use]
extern crate scribe;

extern crate spellbook;



mod compiler;
mod lexer;
mod parser;
mod reader;
mod util;



use std::path::PathBuf;

use spellbook::{EMPTY_STRING, ArgParser, ArgOption};

use ::compiler::Compiler;



const PROGRAM_NAME: &'static str = "Draconic Pass Through Compiler";
const PROGRAM_COMMAND_NAME: &'static str = "draconicpt";
const PROGRAM_VERSION: &'static str = "0.1.0";
const PROGRAM_SUMMARY: &'static str =
   "Compiles a Draconic template to a file.";
const PROGRAM_DESCRIPTION: &'static str =
   "The Draconic HTML Compiler compiles a Draconic template to a file.";



///
fn configure_arguments(parser: &mut ArgParser)
{
   let mut option: ArgOption;

   option = ArgOption::new();
   option.set_name("version");
   option.set_short_name('v');
   option.set_description("Print the programs version information.");
   option.set_optional(true);
   parser.add_option(option);

   option = ArgOption::new();
   option.set_name("input");
   option.set_short_name('i');
   option.set_description("The file to use as input.");
   option.set_arg_description("FILENAME");
   parser.add_option(option);

   option = ArgOption::new();
   option.set_name("output");
   option.set_short_name('o');
   option.set_description("The file to use as output.");
   option.set_arg_description("FILENAME");
   parser.add_option(option);

   option = ArgOption::new();
   option.set_name("include_dir");
   option.set_short_name('I');
   option.set_description("A directory to search when looking for \
                           included files.");
   option.set_arg_description("DIRECTORY_PATH");
   option.set_optional(true);
   parser.add_option(option);
}

///
fn print_version()
{
   println!("Program name: {}", PROGRAM_NAME);
   println!("Version: {}", PROGRAM_VERSION);
   println!("{} {} {} {}", "\u{00A9}", "2016",
            "CyberMages LLC", "All Rights Reserved");
}

///
pub fn main()
{
   let version: bool;
   let input: String;
   let output: String;
   let include_dir: String;
   let mut parser: ArgParser;
   let mut compiler: Compiler;

   // Create a new argument parser.
   parser = ArgParser::new();
   parser.set_name(PROGRAM_COMMAND_NAME);
   parser.set_summary(PROGRAM_SUMMARY);
   parser.set_description(PROGRAM_DESCRIPTION);

   // Configure the arguments that this program takes.
   configure_arguments(&mut parser);

   // Parse the arguments to this program.
   parser.parse();

   // Get the argument values.
   version = parser.get_option_value_as("version", false);
   input = parser.get_option_value_as("input", String::new());
   output = parser.get_option_value_as("output", String::new());
   include_dir = parser.get_option_value_as("include_dir", String::new());

   // If the version is desired then print it instead of running the program.
   if version == true
   {
      print_version();
   }
   else
   {
      // Compile to an HTML file.
      // Create a new compiler.
      compiler = Compiler::new();

      // Register the include directory if there is one.
      if include_dir != EMPTY_STRING
      {
         compiler.register_include_dir(&PathBuf::from(include_dir));
      }

      // Compile the file.
      println!("Compiling {} into {}.", input, output);
      compiler.compile(PathBuf::from(input), PathBuf::from(output));
   }
}
